#### Function: measures the bilateral symmetry of plant leaves in bulk ####
#### Development time: in mid- and late October, 2017 #####################
#### Revision time: March, 2018 ########################################### 
#### Developer: Peijian Shi (PhD) #########################################
#### E-mail: peijianshi@gmail.com #########################################
#### Package 'spatstat' (version >= 1.43-0.025) is needed. ################


#### Function 1: the probability density function 
####                 of the log-logistic distribution 
pdf.fun <- function(P, z){
    b  <- P[1]
    a <- P[2]
    c <- P[3]
    b/a*((z-c)/a)^(b-1)*(1+((z-c)/a)^b)^(-2)
}




#### Function 2: the cumulative density function 
####                 of the log-logistic distribution 
cdf.fun <- function(P, z){
  b  <- P[1]
  a <- P[2]
  c <- P[3]
  ( 1 + (a/(z-c))^b )^(-1)
}




#### Function 3: the inverse function of the cumulative density function
####                 of the log-logistic distribution 
cdf.inverse <- function(P, prob){
    b <- P[1]
    a <- P[2]
    c <- P[3]
    a/(1/prob - 1)^(1/b) + c
}




### Function 4: estimates the parameters of the log-logistic distribution function
log.logistic <- function(x, xlab="x", fig.opt=TRUE, fig.legend=1, tif.format=TRUE){ 
  y <- sort( x )
  N <- length(y)
  F <- c()
  for(i in 1:N){
    F <- c(F, (i-0.35)/N)
  }

  w0 <- 1/N*sum( (1-F)^0 * y )
  w1 <- 1/N*sum( (1-F)^1 * y )
  w2 <- 1/N*sum( (1-F)^2 * y )
  w3 <- 1/N*sum( (1-F)^3 * y )

  b0 <- (2*w1-w0)/(6*w1-w0-6*w2)
  a0 <- (w0-2*w1)*b0 / (gamma(1+1/b0)*gamma(1-1/b0))
  c0 <- w0-a0*gamma(1+1/b0)*gamma(1-1/b0)

  x.int <- (max(x)[1] - min(x)[1])/10
  xmin  <- min(x)[1] - x.int
  xmax  <- max(x)[1] + x.int
  xx    <- seq(xmin, xmax, len=2000)
  pdf.fun <- function(P, z){
    b  <- P[1]
    a <- P[2]
    c <- P[3]
    b/a*((z-c)/a)^(b-1)*(1+((z-c)/a)^b)^(-2)
  }

  cdf.fun <- function(P, z){
    b <- P[1]
    a <- P[2]
    c <- P[3]
    ( 1 + (a/(z-c))^b )^(-1)
  }
  
  value1 <- pdf.fun(c(b0, a0, c0), z=xx)
  value2 <- cdf.fun(c(b0, a0, c0), z=xx)
  # ymax2  <- max(value1)[1]
  # res    <- hist(x, freq=FALSE, plot=FALSE)$density 
  # ymax1  <- max( res )[1]
  # ymax   <- max(ymax1, ymax2)

  if(tif.format=="TRUE" | tif.format=="T"){
    tiff(file=paste("Fig/", fig.legend, " PDF.tif", sep=""), family="serif",
         width=72, height=72, units='mm', res=900, compression='lzw', pointsize=6)
  }
  else{    
    dev.new()
  }
  par(mar=c(5,5,2,2))
  hist( x, freq=FALSE, main="", xlab=xlab, ylim=c(0, 1.25*max(value1)[1]),
        ylab=expression(bold("Probability density function")),
        cex.lab=1.5, cex.axis=1.5, col="grey70" )
  lines(xx, value1, col=2, lwd=2)
  box()
  if(tif.format=="TRUE" | tif.format=="T"){
    dev.off()
  }

  if(tif.format=="TRUE" | tif.format=="T"){
    tiff(file=paste("Fig/", fig.legend, " CDF.tif", sep=""),  family="serif",
         width=72, height=72, units='mm', res=900, compression='lzw',pointsize=6)
  }
  else{    
    dev.new()
  }
  par(mar=c(5,5,2,2))
  plot( ecdf(x), verticals=TRUE, do.p=FALSE, main="",  
        xlab=xlab, ylab=expression(bold("Cumulative distribution function")),
        cex.lab=1.5, cex.axis=1.5, ylim=c(0, 1), col="grey70", lwd=2 )
  lines(xx, value2, col=2, lwd=2)
  if(tif.format=="TRUE" | tif.format=="T"){
    dev.off()
  }

  return( list(par=c(b0, a0, c0)) )

}




#### Function 5: measures the bilateral symmetrical degree of a leaf
bilat.measure <- function( x, y, strip.num=200, animation=TRUE, fig.opt=TRUE, fig.legend=1,   
                           unit="cm", tif.format=FALSE, time.interval=0.001, log.logistic=FALSE, 
                           auto.search=TRUE, ul.np=8000, len.pro=1/20, phi=0, seed=123){
  

 
  if(strip.num / floor(strip.num) != 1 | strip.num <= 0){
    stop("'strip.num' must be a positive integer")
  }

  start.mat  <- cbind(x,y)
  start.mat  <- na.omit(start.mat)
  x          <- start.mat[,1]
  y          <- start.mat[,2]
  x.max      <- max( abs(x) )[1]
  y.max      <- max( abs(y) )[1]
  x          <- x + x.max
  y          <- y + y.max
  poly0      <- as.polygonal( owin(poly=list(x=x, y=y)) )
  data0      <- ppp(x, y, window= poly0) 
  x          <- data0$x
  y          <- data0$y
  if(length(x) >= ul.np){
      set.seed( seed )
      myind <- sample(1:length(x), ul.np, replace=FALSE)
      myind <- sort(myind)
      x     <- x[myind]
      y     <- y[myind]
      poly0 <- as.polygonal( owin(poly=list(x=x, y=y)) )
      data0 <- ppp(x, y, window= poly0)   
  }
  dis.mat    <- pairdist(data0)
  len.max    <- max(dis.mat)[1]
  diag(dis.mat) <- NA 
  adj.mat    <- matrix(NA, nrow=data0$n, ncol=2)
  data1      <- cbind(x=data0$x, y=data0$y)
  dis.pool   <- c()
  for(i in 1:data0$n){
    print(i)
    
    if(i == 1){
      temp <- dis.mat[i, ]
      inde <- 1
    }
    old.inde     <- inde
    temp         <- dis.mat[inde, ]  
    inde         <- which(temp == min(temp[!is.na(temp)]))[1]
    adj.mat[i, ] <- c( data1[old.inde, 1], data1[old.inde, 2] )
    dis.mat[old.inde, ] <- NA
    dis.mat[ ,old.inde] <- NA

    temp.dis     <- min(temp[!is.na(temp)])[1]
    if(temp.dis > len.pro*len.max){
        adj.mat[i, ]    <- adj.mat[i-1, ]
        dis.mat[inde, ] <- NA
        dis.mat[, inde] <- NA
        inde            <- old.inde
    }        

  }

  adj.mat <- unique(adj.mat)
  x       <- adj.mat[,1]
  y       <- adj.mat[,2]  

  #### ֹ(x,y)˶˳ʱ
  #### Ϊas.polygonalĬϵĵ˶ʱ
  poly.temp <- NULL
  try(poly.temp <- as.polygonal( owin(poly=list(x=x, y=y )) ), silent=TRUE)
  if( is.null(poly.temp) ){   
    x       <- x[length(x):1]
    y       <- y[length(y):1]
    poly0   <- as.polygonal( owin(poly=list(x=x, y=y )) )
  }
  if( !is.null(poly.temp) ){
    poly0   <- as.polygonal( owin(poly=list(x=x, y=y )) )
  }

  #### ±ߵĴǱ
  data0   <- ppp(x, y, window=poly0)
  x       <- data0$x
  y       <- data0$y
  n       <- length(x)
  mat     <- pairdist(data0) 

  xlabel <- bquote( bold(paste(bolditalic(x), " (", .(unit), ")", sep="")) ) 
  ylabel <- bquote( bold(paste(bolditalic(y), " (", .(unit), ")", sep="")) )

  if(auto.search != "TRUE" & auto.search != "T"){
    # dev.new()
    par(mar=c(5,5,2,2))
    plot( x, y, asp=1, type="l", xlab=xlabel, ylab=ylabel, 
          cex.lab=1.5, cex.axis=1.5, lwd=3, col="grey50" )
    resu  <- locator(n = 2, type="o", col=2, cex=1.5)
    #### (x1, y1) represents the location of leaf apex
    #### (x2, y2) represents the location of leaf base
    x1    <- resu$x[1]
    x2    <- resu$x[2]
    y1    <- resu$y[1]
    y2    <- resu$y[2]
    if(x1 < x2 | y1 < y2){
      stop("\n Leaf apex must be on the top right corner of leaf base! 
          \n The angle from leaf apex to base should range from 0 to pi/2!
          \n The location of leaf apex should be chosen earlier than that of leaf base!")
    }

    #### To find a point with the shortest distance from any a chosen point
    #### Two points on the edge will be found corresponding to apex and base
    dis.fun1 <- function(z){
      sqrt((z[1]-x1)^2 + (z[2]-y1)^2)
    }
    pp       <- cbind(x,y)
    dis.val1 <- apply(pp, 1L, dis.fun1)
    ind1     <- which(dis.val1==min(dis.val1)[1])[1]
    x1       <- pp[ind1,1]
    y1       <- pp[ind1,2]

    dis.fun2 <- function(z){
      sqrt((z[1]-x2)^2 + (z[2]-y2)^2)
    }

    dis.val2 <- apply(pp, 1L, dis.fun2)
    ind2     <- which(dis.val2==min(dis.val2)[1])[1]
    x2       <- pp[ind2,1]
    y2       <- pp[ind2,2]

  ####################################################################
  }


  if(auto.search == "TRUE" | auto.search == "T"){

    # To find the points corresponding to leaf apex and leaf base
    qq <- cbind( which(mat == max(mat)[1], arr.ind=TRUE) )
    q  <- as.numeric( qq ) 
    x1 <- x[q[1]]
    y1 <- y[q[1]]
    x2 <- x[q[2]]
    y2 <- y[q[2]]
 
  } 

  length0   <- sqrt((x1-x2)^2+(y1-y2)^2)
  length0   <- length0[[1]]

  x      <- x-x2
  y      <- y-y2
  x1     <- x1-x2
  y1     <- y1-y2
  x2     <- x2-x2
  y2     <- y2-y2
  z      <- sqrt( (y2-y1)^2 + (x2-x1)^2 )
  theta  <- acos( (x1-x2)/z ) %% (2*pi)
  x.new  <- x*cos(theta) + y*sin(theta)
  y.new  <- y*cos(theta) - x*sin(theta)

  x.max  <- max(x.new)[1]
  x.min  <- min(x.new)[1]
  y.max  <- max(y.new)[1]
  y.min  <- min(y.new)[1]

  upper.win  <- owin( c(x.min, x.max), c(0, y.max) )
  lower.win  <- owin( c(x.min, x.max), c(y.min, 0) )
  total.poly <- as.polygonal( owin(poly=list(x=x.new, y=y.new)) )
  poly1      <- intersect.owin(upper.win, total.poly)
  poly2      <- intersect.owin(lower.win, total.poly) 

  n     <- nrow(data.frame(total.poly))
  n1    <- nrow(data.frame(poly1))
  n2    <- nrow(data.frame(poly2)) 

  upper.area <- area.owin(poly1)
  lower.area <- area.owin(poly2)
  area0      <- area.owin( union.owin(poly1, poly2) )
  y.max      <- max( c(abs(y.max), abs(y.min)) )[1] 
  x.pred     <- seq( x.min, x.max, len=(strip.num+1) )

  part.upper.area <- c()
  part.lower.area <- c()
  widt            <- c()
  for(j in 2:(strip.num+1)){
    x.temp <- c(x.pred[j-1], x.pred[j])
    um.win <- owin( x.temp, c(0, y.max) ) 
    lm.win <- owin( x.temp, c(-y.max, 0) )
    m.area <- area.owin(um.win)
    u.conv <- setminus.owin(um.win, poly1)
    u.area <- m.area - area.owin(u.conv)
    l.conv <- setminus.owin(lm.win, poly2)
    l.area <- m.area - area.owin(l.conv)
    part.upper.area <- c(part.upper.area, u.area)
    part.lower.area <- c(part.lower.area, l.area)
    temp.u <- max(um.win$y)[1]
    temp.l <- min(lm.win$y)[1]
  }
  D        <- part.upper.area - part.lower.area
  width0   <- max(temp.u-temp.l)[1]
  bi.test  <- wilcox.test(part.upper.area, part.lower.area, paired=TRUE)
  abs.D    <- abs(D)
  SI       <- sum(abs.D/(part.upper.area+part.lower.area))/strip.num
  AR       <- upper.area / lower.area

  Xlab <- bquote(bold(paste("Difference (", .(unit), ''^'2'*')', sep=""))) 


  if( animation == "TRUE" | animation == "T" ){
    if(phi %% (2*pi) != 0){
      xx.new  <- x.new*cos(phi) - y.new*sin(phi)
      yy.new  <- y.new*cos(phi) + x.new*sin(phi)   
    }
    else{
      xx.new  <- x.new
      yy.new  <- y.new
    }   

    if(TRUE){
      # dev.new()
      par(mar=c(5,5,2,2))
      plot( xx.new, yy.new, type="n", xlab=xlabel, ylab=ylabel, cex=0.1, pch=16,
          cex.lab=1.5, cex.axis=1.5, asp=1)
      for(i in 1:length(xx.new)){
        Sys.sleep(time.interval)
        points(xx.new[i], yy.new[i], cex=0.1, col="grey70", pch=16)
      }
      polygon(xx.new, yy.new, col = "grey90", border = "grey70")
      abline(v=0, col=4, lwd=1, lty=2)
      abline(h=0, col=4, lwd=1, lty=2)
      if(phi %% (pi/2) != 0 ){
        abline(0, tan(phi), col=2, lwd=1, lty=2)
      }
      lines(xx.new, yy.new, lwd=1, col="grey70")
    }

    if( tif.format=="TRUE" | tif.format=="T" ){
      tiff(file=paste("Fig/", fig.legend, " LeafEdge.tif", sep=""), family="serif", 
           width=72, height=72, units='mm', res=900, compression='lzw',pointsize=6)
      par(mar=c(5,5,2,2))
      plot( xx.new, yy.new, type="n", xlab=xlabel, ylab=ylabel, cex=0.1, pch=16,
            cex.lab=1.5, cex.axis=1.5, asp=1)
      polygon(xx.new, yy.new, col = "grey90", border = "grey70")
      abline(v=0, col=4, lwd=0.5, lty=2)
      abline(h=0, col=4, lwd=0.5, lty=2)   
      if(phi %% (pi/2) != 0 ){
        abline(0, tan(phi), col=2, lwd=0.5, lty=2)
      }
      lines(xx.new, yy.new, lwd=1, col="grey70")
      dev.off()
    }
  }
  if(log.logistic == "TRUE" | log.logistic == "T"){
      temp <- log.logistic( D, fig.opt=fig.opt, fig.legend=fig.legend, 
                            xlab=Xlab, tif.format=tif.format )
      par  <- temp$par      
  }

  if(fig.opt=="TRUE" | fig.opt=="T"){
    if(tif.format=="TRUE" | tif.format=="T"){
      tiff(file=paste("Fig/", fig.legend, "StripAreaDiff.tif", sep=""),   
           family="serif", width=72, height=72, units='mm', res=900, 
           compression='lzw',pointsize=6)
    }
    else{    
      dev.new()
    }
    par(mar=c(5, 5, 2, 2))
    colors <- ifelse(D >= 0, "blue", "red")
    plot( D, type="h", ylab=Xlab, cex.lab=1.5, 
          cex.axis=1.5, lwd=1, col=colors )
    abline(h=0, col="black", lwd=1)
    if(tif.format=="TRUE" | tif.format=="T"){
      dev.off()
    }
  }

  on.exit(set.seed(seed)) 

  list( n1=n1, n2=n2, n=n, total.poly=total.poly, upper.poly=poly1, 
        lower.poly=poly2, part.upper.area=part.upper.area,  
        part.lower.area=part.lower.area, D=D,   
        scan.area=area0, upper.area=upper.area, lower.area=lower.area, 
        scan.length=length0, scan.width=width0, SI=SI, AR=AR, bi.test=bi.test ) 

}




# creats a folder named 'myfolder' in C disc as the working directory
setwd("c:/")
dir.create("myfolder")
setwd("c:/myfolder/")
# saves the CSV fild named "edge_data.csv" that saves the planar coordinates 
#   of a leaf's edge into the current working directory;
# the planar coordinates of that leaf were generated by the 'profile' function in Matlab,
# which can be found in Shi et al. (2015) doi: 10.3389/fpls.2015.00856
   
dir.create("Fig") 

# Before this, package 'spatstat' (version >= 1.43-0.025) must be installed.
library(spatstat)

#### The angle between the straight line (passing by leaf apex and
####   base) and the x-axis must range 0 to pi/2, which means that
#### leaf apex is on the top right corner of leaf base
#### The location of leaf apex should be chosen earlier than that of leaf base


setwd("c:/Proc/tuliptree") 
dir.create("Fig") 

# Before this, package 'spatstat' (version >= 1.43-0.025) must be installed.
library(spatstat)

#### The angle between the straight line (passing by leaf apex and
####   base) and the x-axis must range 0 to pi/2, which means that
#### leaf apex is on the top right corner of leaf base
#### The location of leaf apex should be chosen earlier than that of leaf base

data <- read.csv("edge_data.csv", header=FALSE)
data <- rbind(data)
X    <- data[,1]
Y    <- data[,2]



H <- bilat.measure( x=X, y=Y, strip.num=1000, animation=TRUE, ul.np=5000,
                    fig.opt=TRUE, fig.legend="Shi", unit="cm", 
                    tif.format=TRUE, log.logistic=FALSE, auto.search=F, 
                    time.interval=0.00045, len.pro=1/10, phi=pi/4 )

results <- data.frame( n1=H$n1, n2=H$n2, n=H$n, scan.length=H$scan.length, scan.width=H$scan.width,  
                       scan.area=H$scan.area, upper.area=H$upper.area, lower.area=H$lower.area, 
                       SI=H$SI, AR=H$AR )

cat('\n')
print( results )
cat('\n')

# Then the results can be saved in the current working directory
write.csv(results, file="results.csv")








